require 'spec_helper'

describe 'afs', type: 'class' do
  on_supported_os.each do |os, facts|
    context "on #{os}" do
      let(:facts) do
        facts
      end
      let(:hiera_config) { 'spec/hiera.yaml' }

      context 'with openafsversion including cern' do
        let(:facts) do
          super().merge(
            openafsversion: '1.6.9-2.el7.cern',
            kernelrelease: '3.10.0-327.28.3.el7.x86_64'
          )
        end

        it { is_expected.to compile.with_all_deps }
        it { is_expected.to contain_package('openafs') }

        it { is_expected.not_to contain_pam('Remove pam_afs_session to ssh stack') }
        it { is_expected.not_to contain_exec('enable_aklog_service') }
        it { is_expected.to contain_service('openafs-client') }
        case facts[:os]['release']['major']
        when '6'
          it { is_expected.to contain_package('kernel-module-openafs-3.10.0-327.28.3.el7.x86_64') }
          it { is_expected.not_to contain_class('systemd') }
        else
          it { is_expected.to contain_package('kmod-openafs') }
          it { is_expected.to contain_class('systemd') }
        end
      end

      context 'with openafsversion not including cern' do
        let(:facts) do
          super().merge(
            openafsversion: '1.6.9-2.el7',
            kernelrelease: '3.10.0-327.28.3.el7.x86_64'
          )
        end

        it { is_expected.to compile.with_all_deps }
        it { is_expected.to contain_package('openafs') }
        it { is_expected.to contain_package('kmod-openafs') }
        it { is_expected.to contain_service('openafs-client') }
        it { is_expected.to contain_file('/etc/sysconfig/openafs').with_content(%r{^AFS_POST_INIT="?/usr/bin/.*$}) }

        context 'with memcache option set to string and too small' do
          let(:params) do
            {
              memcachesize: '1234',
              options: { 'memcache' => true }
            }
          end

          it { is_expected.to compile.with_all_deps }
          # filename would change for 'cern' openafsversion, hence have this in here
          it { is_expected.to contain_file('/etc/sysconfig/openafs').with_content(%r{^AFSD_ARGS=".*-memcache .*"$}) }
          it { is_expected.to contain_file('/usr/vice/etc/cacheinfo').with_content(%r{^/afs:/usr/vice/cache:10240$}) }
        end
        context 'with memcache option set to int and too small' do
          let(:params) do
            {
              memcachesize: 5435,
              options: { 'memcache' => true }
            }
          end

          it { is_expected.to compile.with_all_deps }
          # filename would change for 'cern' openafsversion, hence have this in here
          it { is_expected.to contain_file('/etc/sysconfig/openafs').with_content(%r{^AFSD_ARGS=".*-memcache .*"$}) }
          it { is_expected.to contain_file('/usr/vice/etc/cacheinfo').with_content(%r{^/afs:/usr/vice/cache:10240$}) }
        end

        context 'with memcache option set to string and set okay' do
          let(:params) do
            {
              memcachesize: '12345',
              options: { 'memcache' => true }
            }
          end

          it { is_expected.to compile.with_all_deps }
          # filename would change for 'cern' openafsversion, hence have this in here
          it { is_expected.to contain_file('/etc/sysconfig/openafs').with_content(%r{^AFSD_ARGS=".*-memcache .*"$}) }
          it { is_expected.to contain_file('/usr/vice/etc/cacheinfo').with_content(%r{^/afs:/usr/vice/cache:12345$}) }
        end
        context 'with memcache option set to int and okay size' do
          let(:params) do
            {
              memcachesize: 54_321,
              options: { 'memcache' => true }
            }
          end

          it { is_expected.to compile.with_all_deps }
          # filename would change for 'cern' openafsversion, hence have this in here
          it { is_expected.to contain_file('/etc/sysconfig/openafs').with_content(%r{^AFSD_ARGS=".*-memcache .*"$}) }
          it { is_expected.to contain_file('/usr/vice/etc/cacheinfo').with_content(%r{^/afs:/usr/vice/cache:54321$}) }
        end

        context 'with cachesize option set to string and too small' do
          let(:params) do
            {
              cachesize: '1234'
            }
          end

          it { is_expected.to compile.with_all_deps }
          # filename would change for 'cern' openafsversion, hence have this in here
          it { is_expected.to contain_file('/etc/sysconfig/openafs').without_content(%r{^AFSD_ARGS=".*-cache .*"$}) }
          it { is_expected.to contain_file('/usr/vice/etc/cacheinfo').with_content(%r{^/afs:/usr/vice/cache:131072$}) }
        end
        context 'with cachesize option set to int and too small' do
          let(:params) do
            {
              cachesize: 5435
            }
          end

          it { is_expected.to compile.with_all_deps }
          # filename would change for 'cern' openafsversion, hence have this in here
          it { is_expected.to contain_file('/etc/sysconfig/openafs').without_content(%r{^AFSD_ARGS=".*-memcache .*"$}) }
          it { is_expected.to contain_file('/usr/vice/etc/cacheinfo').with_content(%r{^/afs:/usr/vice/cache:131072$}) }
        end

        context 'with cachesize option set to string and set okay' do
          let(:params) do
            {
              cachesize: '9876543'
            }
          end

          it { is_expected.to compile.with_all_deps }
          # filename would change for 'cern' openafsversion, hence have this in here
          it { is_expected.to contain_file('/etc/sysconfig/openafs').without_content(%r{^AFSD_ARGS=".*-memcache .*"$}) }
          it { is_expected.to contain_file('/usr/vice/etc/cacheinfo').with_content(%r{^/afs:/usr/vice/cache:9876543$}) }
        end
        context 'with cachesize option set to int and okay size' do
          let(:params) do
            {
              cachesize: 9_876_543
            }
          end

          it { is_expected.to compile.with_all_deps }
          # filename would change for 'cern' openafsversion, hence have this in here
          it { is_expected.to contain_file('/etc/sysconfig/openafs').without_content(%r{^AFSD_ARGS=".*-memcache .*"$}) }
          it { is_expected.to contain_file('/usr/vice/etc/cacheinfo').with_content(%r{^/afs:/usr/vice/cache:9876543$}) }
        end

        context 'with cachesize AUTOMATIC 0.8 of the openafspartsize fact' do
          let(:params) do
            {
              cachesize: 'AUTOMATIC'
            }
          end
          let(:facts) do
            super().merge(
              openafspartsize: 1_000_000
            )
          end

          it { is_expected.to compile.with_all_deps }
          # filename would change for 'cern' openafsversion, hence have this in here
          it { is_expected.to contain_file('/etc/sysconfig/openafs').without_content(%r{^AFSD_ARGS=".*-memcache .*"$}) }
          it { is_expected.to contain_file('/usr/vice/etc/cacheinfo').with_content(%r{^/afs:/usr/vice/cache:800000$}) }
        end
        context 'with cachesize AUTOMATIC 0.8 and no openafspartsize fact' do
          let(:params) do
            {
              cachesize: 'AUTOMATIC'
            }
          end

          it { is_expected.to compile.with_all_deps }
          # filename would change for 'cern' openafsversion, hence have this in here
          it { is_expected.to contain_file('/etc/sysconfig/openafs').without_content(%r{^AFSD_ARGS=".*-memcache .*"$}) }
          it { is_expected.to contain_file('/usr/vice/etc/cacheinfo').with_content(%r{^/afs:/usr/vice/cache:131072$}) }
        end
      end

      context 'with enable_firewall false' do
        let(:params) do
          {
            enable_firewall: false
          }
        end

        it { is_expected.to compile.with_all_deps }
        it { is_expected.to have_firewall_resource_count(0) }
        it { is_expected.to have_firewalld_rich_rule_resource_count(0) }
        it { is_expected.not_to contain_class('nftables') }
      end

      context 'with enable_firewall unset' do
        case facts[:os]['release']['major']
        when '6', '7'
          it { is_expected.to have_firewalld_rich_rule_resource_count(0) }
          it { is_expected.not_to contain_class('nftables') }
          it { is_expected.to have_pam_resource_count(0) }
          it { is_expected.to contain_firewall('100 allow afs3-callback') }
          it { is_expected.not_to contain_class('afs::kojitag') }
          it { is_expected.to contain_package('pam_krb5') }
          it { is_expected.not_to contain_package('pam_afs_session') }
          it { is_expected.not_to contain_package('cern-aklog-systemd-user') }
          it { is_expected.not_to contain_file('/usr/local/bin/aklog.sh') }
        else
          it { is_expected.to have_firewall_resource_count(0) }
          it { is_expected.to contain_class('afs::kojitag') }
          it { is_expected.not_to contain_package('pam_krb5') }
          it { is_expected.to contain_file('/usr/local/bin/aklog.sh').with_content(%r{^exec /usr/bin/aklog \$@$}) }
          it { is_expected.to contain_package('pam_afs_session') }
          it { is_expected.to contain_pam('Add pam_afs_session to ssh stack') }
          it { is_expected.to contain_package('cern-aklog-systemd-user').with_ensure('absent') }
        end
      end

      context 'with cernfw::manager set to iptables' do
        let(:pre_condition) { 'class{"cernfw": manager => "iptables"}' }

        it { is_expected.to contain_firewall('100 allow afs3-callback') }
        it { is_expected.to have_firewalld_rich_rule_resource_count(0) }
        it { is_expected.not_to contain_class('nftables::rules::afs3_callback') }
      end

      context 'with cernfw::manager set to firewalld' do
        let(:pre_condition) { 'class{"cernfw": manager => "firewalld"}' }

        it { is_expected.to have_firewall_resource_count(0) }
        it { is_expected.to contain_firewalld_rich_rule('AFS afs3-callback') }
        it { is_expected.not_to contain_class('nftables::rules::afs3_callback') }
      end

      context 'with cernfw::manager set to nftables' do
        let(:pre_condition) { 'class{"cernfw": manager => "nftables"}' }

        it { is_expected.to have_firewall_resource_count(0) }
        it { is_expected.to have_firewalld_rich_rule_resource_count(0) }
        it { is_expected.to contain_class('nftables::rules::afs3_callback') }
      end

      context 'with aklog_unit and pam_afs_session true' do
        let(:params) do
          {
            aklog_unit: true,
            pam_afs_session: true
          }
        end

        it { is_expected.not_to contain_pam('Remove pam_afs_session to ssh stack') }
        case facts[:os]['release']['major']
        when '6', '7'
          it { is_expected.not_to contain_package('pam_afs_session') }
          it { is_expected.not_to contain_pam('Add pam_afs_session to ssh stack') }
          it { is_expected.not_to contain_package('cern-aklog-systemd-user') }
          it { is_expected.not_to contain_exec('enable_aklog_service') }
        else
          it { is_expected.to contain_package('pam_afs_session').with_ensure('present') }
          it { is_expected.to contain_pam('Add pam_afs_session to ssh stack') }
          it { is_expected.to contain_package('cern-aklog-systemd-user').with_ensure('present') }
          it { is_expected.to contain_exec('enable_aklog_service') }
        end
      end

      context 'with aklog_unit and pam_afs_session false' do
        let(:params) do
          {
            aklog_unit: false,
            pam_afs_session: false
          }
        end

        it { is_expected.not_to contain_exec('enable_aklog_service') }
        it { is_expected.not_to contain_pam('Add pam_afs_session to ssh stack') }
        case facts[:os]['release']['major']
        when '6', '7'
          it { is_expected.not_to contain_pam('Remove pam_afs_session to ssh stack') }
          it { is_expected.not_to contain_package('pam_afs_session') }
          it { is_expected.not_to contain_package('cern-aklog-systemd-user') }
        else
          it { is_expected.to contain_package('pam_afs_session').with_ensure('absent') }
          it { is_expected.to contain_package('cern-aklog-systemd-user').with_ensure('absent') }
          it { is_expected.to contain_pam('Remove pam_afs_session to ssh stack') }
        end
      end

      context 'with handle_cellservdb not set' do
        it { is_expected.to contain_file('/usr/vice/etc/CellServDB.dist') }
        it { is_expected.to contain_file('/usr/vice/etc/CellServDB.local') }
      end

      context 'with handle_cellservdb false' do
        let(:params) do
          {
            handle_cellservdb: false
          }
        end

        it { is_expected.not_to contain_file('/usr/vice/etc/CellServDB.dist') }
        it { is_expected.not_to contain_file('/usr/vice/etc/CellServDB.local') }
      end
    end
  end
end
